@@ -14,7 +14,7 @@ from yapic import json
 from cryptofeed.backends._util import book_flatten
 from cryptofeed.backends.backend import (BackendBookCallback, BackendBookDeltaCallback, BackendCandlesCallback, BackendFundingCallback,
                                          BackendOpenInterestCallback, BackendTickerCallback, BackendTradeCallback,
-                                         BackendLiquidationsCallback, BackendMarketInfoCallback, BackendTransactionsCallback)
+                                         BackendLiquidationsCallback, BackendMarketInfoCallback)
 from cryptofeed.backends.http import HTTPCallback
 
 
@@ -106,9 +106,5 @@ class MarketInfoElastic(ElasticCallback, BackendMarketInfoCallback):
     default_index = 'market_info'
 
 
-class TransactionsElastic(ElasticCallback, BackendTransactionsCallback):
-    default_index = 'transactions'
-
-
 class CandlesElastic(ElasticCallback, BackendCandlesCallback):
     default_index = 'candles'
