@@ -12,7 +12,7 @@ from yapic import json
 
 from cryptofeed.backends.backend import (BackendCandlesCallback, BackendQueue, BackendBookCallback, BackendBookDeltaCallback, BackendFundingCallback,
                                          BackendOpenInterestCallback, BackendTickerCallback, BackendTradeCallback,
-                                         BackendLiquidationsCallback, BackendMarketInfoCallback, BackendTransactionsCallback)
+                                         BackendLiquidationsCallback, BackendMarketInfoCallback)
 
 
 LOG = logging.getLogger('feedhandler')
@@ -134,9 +134,5 @@ class MarketInfoSocket(SocketCallback, BackendMarketInfoCallback):
     default_key = 'market_info'
 
 
-class TransactionsSocket(SocketCallback, BackendTransactionsCallback):
-    default_key = 'transactions'
-
-
 class CandlesSocket(SocketCallback, BackendCandlesCallback):
     default_key = 'candles'
